// traceSDK import must come before anything else, do not import any functional library before this!
import traceSDK from './src/instrumentation';

import logger, {loggingMiddleware} from "./src/logger";
import express, {Express, NextFunction} from "express";
import {RowDataPacket, createPool, Pool, FieldPacket} from "mysql2/promise";
import bodyParser from "body-parser";
import {createProxyMiddleware} from "http-proxy-middleware";
import {metrics} from "@opentelemetry/api";
import axios from "axios";

const PORT: number = parseInt(process.env.PORT || '8080');
const app: Express = express();

function getRandomNumber(min: number, max: number) {
    return Math.floor(Math.random() * (max - min) + min);
}

interface IRecord extends RowDataPacket {
    id: number
    updated_at?: Date
    test_field: boolean
}

class RecordRepo {
    connPool: Pool;

    constructor() {
        const connectionOptions = {
            host: process.env.DB_HOST,
            port: parseInt(process.env.DB_PORT as string, 10),
            user: process.env.DB_USER,
            password: process.env.DB_PASS,
        };
        this.connPool = createPool({
            ...connectionOptions,
            database: 'test_db'
        });
    }

    async readByID(id: number): Promise<IRecord | undefined> {
        const [results]: [IRecord[], FieldPacket[]] = await this.connPool.query(
            "SELECT * FROM test_db.test_tab WHERE id = ?;",
            [id],
        );
        if (!results || !results.length)
            return undefined;
        return results[0]
    }

    async upsert(data: IRecord | undefined): Promise<IRecord | undefined> {
        if (!data) return undefined;
        if (!data.id && data.id !== 0) return undefined;
        if (!data.id) {
            await this.connPool.execute(`INSERT INTO test_db.test_tab (updated_at, test_field)
                                         VALUES (?, ?);`,
                [new Date(), Boolean(data.test_field).valueOf()]
            );
        } else {
            await this.connPool.execute(`INSERT INTO test_db.test_tab (id, updated_at, test_field)
                                         VALUES (?, ?, ?)
                                         ON DUPLICATE KEY UPDATE updated_at = VALUES(updated_at),
                                                                 test_field = VALUES(test_field);`,
                [data.id, new Date(), Boolean(data.test_field).valueOf()]
            );
        }
        return undefined;
    }
}

const repo = new RecordRepo();
app.use(bodyParser.json());
app.use(loggingMiddleware);

// error handling
const errorHandler = (req: any, res: any, next: NextFunction) => {
    try {
        next();
    } catch (err: any) {
        console.error(err.stack);
        res.status(500).json({error: "error. check the logs"});
    }
}
app.use(errorHandler);

// Endpoint that writes data into DB
app.post("/data", async (req, res) => {
    await repo.upsert(req.body);
    res.status(200).send("OK");
});

// Endpoint that reads data from DB
app.get("/data", async (req, res) => {
    const result = await repo.readByID(parseInt(req.query.id as string, 10));
    res.status(200).json({
        id: result?.id,
        updated_at: result?.updated_at?.getTime(),
        test_field: !!result?.test_field
    });
});

const meter = metrics.getMeter("dice-lib");
const counter = meter.createCounter('dice-lib.rolls.counter');

// Simple endpoint that has no dependencies
app.get('/rolldice', (req, res) => {
    try {
        const dieResult: number = getRandomNumber(1, 7);
        logger.info(`you rolled ${dieResult.toString()}`);
        counter.add(1, {'result': dieResult});
        if (dieResult === 6) {
            res.status(500).json({error: "a mock internal error"});
        } else {
            res.status(200).json({result: dieResult});
        }
    } catch (e) {
        res.status(500).send(`lambda threw error: ${(e as any).toString()}`);
    }
});

const tmpApiProxy = createProxyMiddleware({
    pathFilter: "/*",
    target: "http://proxied-app:8080",
    pathRewrite: {
        '^/proxy/': '/',
    }
});

const wrappedAPiProxy = function apiProxy(...args: any[]) {
    // @ts-ignore ok
    return tmpApiProxy(...args);
}

app.use('/proxy', wrappedAPiProxy);

// Another simple endpoint that has no dependencies
app.get('/', (req, res) => {
    res.json({status: "OK"});
});

const server = app.listen(PORT, () => {
    logger.info(`Listening for requests on http://localhost:${PORT}`);
});

const shutdown = async () => {
    await Promise.all([
        new Promise(resolve => setTimeout(resolve, 9000)).then(() => logger.info("9s over")),
        repo.connPool.end()
            .then(() => console.log("repo connections closed"))
            .catch((err: any) => console.error("failed shutdown down repository", err)),
        traceSDK.shutdown()
            .then(() => console.log("telemetry flushed"))
            .catch((err: any) => console.error("failed shutdown down tracer", err)),
    ])
    return;
};

// TODO do not use process.on('exit', ...) as it is later in the nodeJS shutdown lifecycle
process.on('SIGTERM', async () => {
    logger.info("app shutting down...");
    await shutdown()
        .then(() => console.log("app shutdown success"))
        .catch(err => console.error("app shutdown failed.", err));
    process.exit(0);
});
