import {api, NodeSDK} from '@opentelemetry/sdk-node';
import {getNodeAutoInstrumentations} from '@opentelemetry/auto-instrumentations-node';
import {MeterProvider, PeriodicExportingMetricReader} from '@opentelemetry/sdk-metrics';
import {HostMetrics} from "@opentelemetry/host-metrics";
import {Resource} from "@opentelemetry/resources";
import * as semConv from "@opentelemetry/semantic-conventions";
import {version} from "../package.json";
import {OTLPTraceExporter} from "@opentelemetry/exporter-trace-otlp-http"; // make sure this is http for best compatibility
import {OTLPMetricExporter} from '@opentelemetry/exporter-metrics-otlp-http';
import {BatchLogRecordProcessor} from "@opentelemetry/sdk-logs";
import {OTLPLogExporter} from "@opentelemetry/exporter-logs-otlp-http";
import {MySQL2InstrumentationConfig} from "@opentelemetry/instrumentation-mysql2";
import {HttpInstrumentationConfig} from "@opentelemetry/instrumentation-http";

const resource = new Resource({
    [semConv.SEMRESATTRS_SERVICE_NAME]: 'otel_poc',
    [semConv.SEMRESATTRS_SERVICE_NAMESPACE]: 'experimentation',
    [semConv.SEMRESATTRS_SERVICE_VERSION]: version,
    [semConv.SEMRESATTRS_DEPLOYMENT_ENVIRONMENT]: 'dev',
    [semConv.SEMRESATTRS_CONTAINER_ID]: process.env.HOSTNAME || 'not_available' // FIXME docker-compose only
});

const logExporter = new OTLPLogExporter();
const logProcessor = new BatchLogRecordProcessor(logExporter);
const metricExporter = new OTLPMetricExporter({});

const sdk = new NodeSDK({
    resource: resource,
    traceExporter: new OTLPTraceExporter({}),
    metricReader: new PeriodicExportingMetricReader({
        exporter: metricExporter,
    }),
    logRecordProcessor: logProcessor,
    instrumentations: [
        getNodeAutoInstrumentations({
            '@opentelemetry/instrumentation-mysql2': {
                responseHook: (span, responseHookInfo) => {
                    // For more info, read
                    // https://github.com/open-telemetry/opentelemetry-js-contrib/issues/1758
                    span.setAttribute(semConv.SEMATTRS_DB_STATEMENT, 'REDACTED')
                }
            } as MySQL2InstrumentationConfig,
            '@opentelemetry/instrumentation-http': {} as HttpInstrumentationConfig
        }),
    ],
});

sdk.start();

const hostMetrics = new HostMetrics({
    name: resource.attributes[semConv.SEMRESATTRS_SERVICE_NAME] as string,
    meterProvider: api.metrics.getMeterProvider() as MeterProvider
});
hostMetrics.start();

export default sdk;
